require 'pg'

require 'common/plugin'
require 'mv/mv_plugin'

class RoundcubeDbMv

  include Plugin
  include MvPlugin

  def initialize()
    cfg = Configuration.new()
    @db_host = cfg.roundcube_dbhost
    @db_port = cfg.roundcube_dbport
    @db_opts = cfg.roundcube_dbopts
    @db_tty  = cfg.roundcube_dbtty
    @db_name = cfg.roundcube_dbname
    @db_user = cfg.roundcube_dbuser
    @db_pass = cfg.roundcube_dbpass
  end


  def describe_domain(domain)
    # Roundcube doesn't have a concept of domains.
    return 'N/A'
  end

  def describe_account(account)
    user_id = self.get_user_id(account)

    if user_id.nil?
      return 'User not found'
    else
      return "User ID: #{user_id}"
    end
  end

  def mv_domain(from, to)
    # Roundcube doesn't have a concept of domains.
  end

  def mv_account(from, to)
    sql_queries = ['UPDATE users SET username = $1 WHERE username = $2;']

    begin
      connection = PGconn.connect(@db_host,
                                  @db_port,
                                  @db_opts,
                                  @db_tty,
                                  @db_name,
                                  @db_user,
                                  @db_pass)

      sql_queries.each do |sql_query|
        connection.query(sql_query, [to, from])
      end

      connection.close()

    rescue PGError => e
      # Pretend like we're database-agnostic in case we ever are.
      raise DatabaseError.new(e)
    end

  end


  protected;

  def get_user_id(account)
    user_id = nil

    begin
      connection = PGconn.connect(@db_host,
                                  @db_port,
                                  @db_opts,
                                  @db_tty,
                                  @db_name,
                                  @db_user,
                                  @db_pass)

      sql_query = "SELECT user_id FROM users WHERE username = $1;"

      connection.query(sql_query, [account]) do |result|
        if result.num_tuples > 0
          user_id = result[0]['user_id']
        end
      end

      connection.close()

    rescue PGError => e
      # Pretend like we're database-agnostic in case we ever are.
      raise DatabaseError.new(e)
    end

    return user_id
  end

end
